package org.zipcoder.store;

import java.util.*;

public class MyHashMap implements MyMap{
    // final field that doesn't change for any object
    private static final int BUCKET_SIZE = 100;

    // instance field that can only be set once. It is different for every Object
    private final List<Entry>[] entries;

    public MyHashMap(){
        entries = new List[BUCKET_SIZE];

        for(int i = 0; i < BUCKET_SIZE; i++) {
            entries[i] = new ArrayList<>();
        }
    }

    @Override
    public int size() {
        int size = 0;
        for (int i = 0; i < BUCKET_SIZE; i++) {
            size += entries[i].size();
        }
        return size;
    }

    @Override
    public boolean isEmptry() {
        return size() == 0;
    }

    @Override
    public Cart get(User key) {
        for (Entry current : entries[bucketIndex(key)]) {
            if (current.getKey().equals(key)) {
                return current.getValue();
            }
        }
        return null;
    }

    @Override
    public void put(User key, Cart value) {
        boolean entryNotAdded = true;
        for (Entry current : entries[bucketIndex(key)]) {
            if (current.getKey().equals(key)) {
                entryNotAdded = false;
                current.setValue(value);
            }
        }
        if (entryNotAdded) {
            Entry entry = new Entry(key, value);
            entries[bucketIndex(key)].add(entry);
        }
    }

    @Override
    public List<User> getKeys() {
        List<User> listOfUsers = new ArrayList<>();
        for (int i = 0; i < BUCKET_SIZE; i++) {
            for (Entry current : entries[i]) {
                listOfUsers.add(current.getKey());
            }
        }
        return listOfUsers;
    }

    @Override
    public List<Cart> getValues() {
        List<Cart> listOfCarts = new ArrayList<>();
        for (int i = 0; i < BUCKET_SIZE; i++) {
            for (Entry current : entries[i]) {
                listOfCarts.add(current.getValue());
            }
        }
        return listOfCarts;
    }

    @Override
    public Cart remove(User key) {
        for (Entry current : entries[bucketIndex(key)]) {
            if (current.getKey().equals(key)) {
                entries[bucketIndex(key)].remove(current);
                return current.getValue();
            }
        }
        return null;
    }

    private int bucketIndex(User key){
        return Math.abs(key.hashCode()) % entries.length;
    }
}
